from __future__ import annotations

from .ara.ArEntail import *
from .ces.CTKFactsNLI import *
from .deu.FalseFriendsDeEnPC import *
from .eng.LegalBenchPC import *
from .eng.SprintDuplicateQuestionsPC import *
from .eng.TwitterSemEval2015PC import *
from .eng.TwitterURLCorpusPC import *
from .fas.FarsTail import *
from .hye.ArmenianParaphrasePC import *
from .ind.IndoNLI import *
from .kor.KlueNLI import *
from .multilingual.OpusparcusPC import *
from .multilingual.PawsX import *
from .multilingual.RTE3 import *
from .multilingual.XNLI import *
from .multilingual.XStance import *
from .pol.PolishPC import *
from .por.Assin2RTE import *
from .por.SickBrPC import *
from .zho.CMTEBPairClassification import *
